package mega.privacy.android.app.main.adapters;

import static android.view.View.GONE;
import static android.view.View.VISIBLE;
import static mega.privacy.android.app.utils.Constants.INVALID_POSITION;
import static mega.privacy.android.app.utils.Constants.THUMB_CORNER_RADIUS_DP;
import static mega.privacy.android.app.utils.ThumbnailUtils.getRoundedBitmap;
import static mega.privacy.android.app.utils.ThumbnailUtils.getThumbnailFromCache;
import static mega.privacy.android.app.utils.ThumbnailUtils.getThumbnailFromFolder;
import static mega.privacy.android.app.utils.Util.dp2px;
import static mega.privacy.android.app.utils.Util.getSizeString;
import static mega.privacy.android.app.utils.Util.getSpeedString;
import static mega.privacy.android.app.utils.Util.isOnline;
import static nz.mega.sdk.MegaTransfer.STATE_ACTIVE;
import static nz.mega.sdk.MegaTransfer.STATE_COMPLETING;
import static nz.mega.sdk.MegaTransfer.STATE_PAUSED;
import static nz.mega.sdk.MegaTransfer.STATE_QUEUED;
import static nz.mega.sdk.MegaTransfer.STATE_RETRYING;
import static nz.mega.sdk.MegaTransfer.TYPE_DOWNLOAD;
import static nz.mega.sdk.MegaTransfer.TYPE_UPLOAD;

import android.app.Activity;
import android.content.Context;
import android.graphics.Bitmap;
import android.util.DisplayMetrics;
import android.util.SparseBooleanArray;
import android.view.Display;
import android.view.LayoutInflater;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.ViewGroup;
import android.view.animation.Animation;
import android.view.animation.AnimationUtils;
import android.widget.ImageView;
import android.widget.RelativeLayout;
import android.widget.TextView;

import androidx.core.content.ContextCompat;
import androidx.recyclerview.widget.RecyclerView;

import java.util.ArrayList;
import java.util.List;

import mega.privacy.android.app.MegaApplication;
import mega.privacy.android.app.MimeTypeList;
import mega.privacy.android.app.R;
import mega.privacy.android.app.globalmanagement.TransfersManagement;
import mega.privacy.android.app.main.ManagerActivity;
import mega.privacy.android.app.utils.ThumbnailUtils;
import nz.mega.sdk.MegaApiAndroid;
import nz.mega.sdk.MegaApiJava;
import nz.mega.sdk.MegaNode;
import nz.mega.sdk.MegaTransfer;
import timber.log.Timber;

public class MegaTransfersAdapter extends RecyclerView.Adapter<MegaTransfersAdapter.ViewHolderTransfer> implements OnClickListener, RotatableAdapter {

    private final Context context;
    private MegaApiAndroid megaApi;

    private final DisplayMetrics outMetrics;

    private List<MegaTransfer> tL;
    private final RecyclerView listFragment;

    private final SelectModeInterface selectModeInterface;
    private boolean multipleSelect;
    private SparseBooleanArray selectedItems;

    private TransfersManagement transfersManagement;

    public MegaTransfersAdapter(Context _context, List<MegaTransfer> _transfers,
                                RecyclerView _listView, SelectModeInterface selectModeInterface,
                                TransfersManagement transfersManagement) {
        this.context = _context;
        this.tL = _transfers;
        this.listFragment = _listView;
        this.selectModeInterface = selectModeInterface;
        this.transfersManagement = transfersManagement;

        if (megaApi == null) {
            megaApi = MegaApplication.getInstance().getMegaApi();
        }

        Display display = ((Activity) context).getWindowManager().getDefaultDisplay();
        outMetrics = new DisplayMetrics();
        display.getMetrics(outMetrics);
    }

    public static class ViewHolderTransfer extends RecyclerView.ViewHolder {
        public ViewHolderTransfer(View v) {
            super(v);
        }

        public ImageView imageView;
        public ImageView iconDownloadUploadView;
        public TextView textViewFileName;
        public ImageView imageViewCompleted;
        public TextView textViewCompleted;
        public RelativeLayout itemLayout;
        public ImageView optionReorder;
        public ImageView optionPause;
        public long document;
        public String currentPath;
        public TextView progressText;
        public TextView speedText;
    }


    public void setTransfers(List<MegaTransfer> transfers) {
        this.tL = transfers;
        notifyDataSetChanged();
    }

    @Override
    public ViewHolderTransfer onCreateViewHolder(ViewGroup parent, int viewType) {
        View v = LayoutInflater.from(parent.getContext()).inflate(R.layout.item_transfers_list, parent, false);

        ViewHolderTransfer holder = new ViewHolderTransfer(v);
        holder.itemLayout = v.findViewById(R.id.transfers_list_item_layout);
        holder.itemLayout.setOnClickListener(this);
        holder.imageView = v.findViewById(R.id.transfers_list_thumbnail);
        holder.iconDownloadUploadView = v.findViewById(R.id.transfers_list_small_icon);
        holder.textViewFileName = v.findViewById(R.id.transfers_list_filename);
        holder.progressText = v.findViewById(R.id.transfers_progress_text);
        holder.speedText = v.findViewById(R.id.transfers_speed_text);
        holder.imageViewCompleted = v.findViewById(R.id.transfers_list_completed_image);
        holder.textViewCompleted = v.findViewById(R.id.transfers_list_completed_text);
        holder.optionReorder = v.findViewById(R.id.transfers_list_option_reorder);
        holder.optionReorder.setOnClickListener(this);
        holder.optionPause = v.findViewById(R.id.transfers_list_option_pause);
        holder.optionPause.setOnClickListener(this);
        v.setTag(holder);

        return holder;
    }

    @Override
    public void onBindViewHolder(ViewHolderTransfer holder, int position) {
        MegaTransfer transfer = (MegaTransfer) getItem(position);

        if (transfer == null) {
            Timber.w("The recovered transfer is NULL - do not update");
            return;
        }

        String fileName = transfer.getFileName();
        holder.textViewFileName.setText(fileName);

        RelativeLayout.LayoutParams params = (RelativeLayout.LayoutParams) holder.imageView.getLayoutParams();
        boolean isItemChecked = isItemChecked(position);
        int transferType = transfer.getType();
        int transferState = transfer.getState();

        if (transferType == TYPE_DOWNLOAD) {
            holder.progressText.setTextColor(ContextCompat.getColor(context, R.color.green_500_green_400));
            holder.document = transfer.getNodeHandle();

            if (!isItemChecked) {
                holder.iconDownloadUploadView.setImageResource(R.drawable.ic_download_transfers);

                MegaNode node = megaApi.getNodeByHandle(transfer.getNodeHandle());
                if (node != null) {
                    RelativeLayout.LayoutParams params3 = (RelativeLayout.LayoutParams) holder.iconDownloadUploadView.getLayoutParams();
                    params3.setMargins(0, 0, 0, 0);
                    holder.iconDownloadUploadView.setLayoutParams(params3);

                    Bitmap thumb = null;
                    if (node.hasThumbnail()) {
                        thumb = getThumbnailFromCache(node);
                        if (thumb == null) {
                            thumb = getThumbnailFromFolder(node, context);
                        }

                        if (thumb == null) {
                            try {
                                thumb = ThumbnailUtils.getThumbnailFromMegaTransfer(node, context, holder, megaApi, this);
                            } catch (Exception e) {
                                Timber.e(e, "Exception getting thumbnail");
                            }
                        }
                    }

                    if (thumb != null) {
                        params.height = params.width = dp2px(36, outMetrics);
                        params.setMargins(54, 0, 18, 0);

                        RelativeLayout.LayoutParams params1 = (RelativeLayout.LayoutParams) holder.iconDownloadUploadView.getLayoutParams();
                        params1.setMargins(0, -12, -12, 0);
                        holder.iconDownloadUploadView.setLayoutParams(params1);

                        holder.imageView.setImageBitmap(getRoundedBitmap(context, thumb, dp2px(THUMB_CORNER_RADIUS_DP)));
                    } else {
                        params.height = params.width = dp2px(48, outMetrics);
                        params.setMargins(36, 0, 0, 0);

                        holder.imageView.setImageResource(MimeTypeList.typeForName(transfer.getFileName()).getIconResourceId());
                    }

                    holder.imageView.setLayoutParams(params);
                }
            }
        } else if (transferType == TYPE_UPLOAD) {
            if (!isItemChecked) {
                holder.iconDownloadUploadView.setImageResource(R.drawable.ic_upload_transfers);
                RelativeLayout.LayoutParams params3 = (RelativeLayout.LayoutParams) holder.iconDownloadUploadView.getLayoutParams();
                params3.setMargins(0, 0, 0, 0);
                holder.iconDownloadUploadView.setLayoutParams(params3);

                holder.imageView.setImageResource(MimeTypeList.typeForName(transfer.getFileName()).getIconResourceId());
                params.height = params.width = dp2px(48, outMetrics);
                params.setMargins(36, 0, 0, 0);
                holder.imageView.setLayoutParams(params);
            }

            holder.progressText.setTextColor(ContextCompat.getColor(context, R.color.dark_blue_500_200));
            holder.currentPath = transfer.getPath();
        }

        if (isItemChecked) {
            holder.iconDownloadUploadView.setVisibility(GONE);
            holder.imageView.setImageResource(R.drawable.ic_select_folder);
            params.height = params.width = dp2px(48, outMetrics);
            params.setMargins(36, 0, 0, 0);
            holder.imageView.setLayoutParams(params);
        } else {
            holder.iconDownloadUploadView.setVisibility(VISIBLE);
        }

        if (isMultipleSelect()) {
            holder.optionReorder.setVisibility(GONE);
            holder.optionPause.setVisibility(GONE);
        } else {
            holder.optionReorder.setVisibility(VISIBLE);
            holder.optionPause.setVisibility(VISIBLE);
            holder.optionPause.setImageResource(R.drawable.ic_pause_grey);
        }

        holder.textViewCompleted.setVisibility(GONE);
        holder.imageViewCompleted.setVisibility(GONE);
        holder.imageViewCompleted.setImageResource(R.drawable.ic_queue);
        holder.progressText.setVisibility(VISIBLE);
        holder.speedText.setVisibility(VISIBLE);

        if (megaApi.areTransfersPaused(MegaTransfer.TYPE_DOWNLOAD) || megaApi.areTransfersPaused(MegaTransfer.TYPE_UPLOAD)) {
            holder.progressText.setText(getProgress(transfer));
            holder.speedText.setText(context.getResources().getString(R.string.transfer_paused));

            if (!isMultipleSelect() && transferState == STATE_PAUSED) {
                holder.optionPause.setImageResource(R.drawable.ic_play_grey);
            }
        } else {
            switch (transferState) {
                case STATE_PAUSED:
                    holder.progressText.setText(getProgress(transfer));
                    holder.speedText.setText(context.getResources().getString(R.string.transfer_paused));

                    if (!isMultipleSelect()) {
                        holder.optionPause.setImageResource(R.drawable.ic_play_grey);
                    }

                    break;

                case STATE_ACTIVE:
                    holder.progressText.setText(getProgress(transfer));
                    holder.speedText.setText(getSpeedString(isOnline(context)
                            ? transfer.getSpeed()
                            : 0));
                    break;

                case STATE_COMPLETING:
                case STATE_RETRYING:
                case STATE_QUEUED:
                    if ((transferType == TYPE_DOWNLOAD && transfersManagement.isOnTransferOverQuota())
                            || (transferType == TYPE_UPLOAD && MegaApplication.getInstance().getStorageState() == MegaApiJava.STORAGE_STATE_RED)) {
                        holder.progressText.setTextColor(ContextCompat.getColor(context, R.color.orange_400_orange_300));

                        if (transferType == TYPE_DOWNLOAD) {
                            holder.progressText.setText(String.format("%s %s", getProgress(transfer), context.getString(R.string.label_transfer_over_quota)));
                        } else {
                            holder.progressText.setText(String.format("%s %s", getProgress(transfer), context.getString(R.string.label_storage_over_quota)));
                        }

                        holder.speedText.setVisibility(GONE);
                    } else if (transferState == STATE_QUEUED) {
                        holder.progressText.setVisibility(GONE);
                        holder.speedText.setVisibility(GONE);
                        holder.imageViewCompleted.setVisibility(VISIBLE);
                        holder.textViewCompleted.setVisibility(VISIBLE);
                        holder.textViewCompleted.setText(context.getResources().getString(R.string.transfer_queued));
                    } else {
                        holder.progressText.setText(getProgress(transfer));

                        if (transferState == STATE_COMPLETING) {
                            holder.speedText.setText(context.getResources().getString(R.string.transfer_completing));
                        } else {
                            holder.speedText.setText(context.getResources().getString(R.string.transfer_retrying));
                        }
                    }
                    break;

                default:
                    Timber.d("Default status");
                    holder.progressText.setVisibility(GONE);
                    holder.speedText.setVisibility(GONE);
                    holder.imageViewCompleted.setVisibility(VISIBLE);
                    holder.textViewCompleted.setVisibility(VISIBLE);
                    holder.textViewCompleted.setText(context.getResources().getString(R.string.transfer_unknown));
                    holder.optionPause.setVisibility(GONE);
                    break;
            }
        }

        holder.itemLayout.setTag(holder);
        holder.optionReorder.setTag(holder);
        holder.optionPause.setTag(holder);
    }

    /**
     * Get the progress of a transfer.
     *
     * @param transfer transfer to get the progress
     * @return The progress of the transfer.
     */
    private String getProgress(MegaTransfer transfer) {
        long percentage = Math.round(100.0 * transfer.getTransferredBytes() / transfer.getTotalBytes());
        String size = getSizeString(transfer.getTotalBytes());

        return context.getString(R.string.progress_size_indicator, percentage, size);
    }

    @Override
    public int getItemCount() {
        return tL.size();
    }

    public Object getItem(int position) {
        if (position >= 0) {
            return tL.get(position);
        }
        Timber.e("Error: position NOT valid: %s", position);
        return null;
    }

    @Override
    public long getItemId(int position) {
        return position;
    }

    @Override
    public void onClick(View v) {
        Timber.d("onClick");

        ViewHolderTransfer holder = (ViewHolderTransfer) v.getTag();
        if (holder == null) {
            Timber.w("Holder is NULL- not action performed");
            return;
        }

        int currentPosition = holder.getAdapterPosition();
        MegaTransfer t = (MegaTransfer) getItem(currentPosition);
        if (t == null) {
            Timber.w("MegaTransfer is NULL- not action performed");
            return;
        }

        if (v.getId() == R.id.transfers_list_option_pause) {
            ((ManagerActivity) context).pauseIndividualTransfer(t);
        } else if (v.getId() == R.id.transfers_list_item_layout && isMultipleSelect()) {
            toggleSelection(currentPosition);
        }
    }

    /**
     * Adds a new transfer to the adapter.
     *
     * @param tL       Updated list of transfers.
     * @param position Position of the transfer in the adapter.
     */
    public void addItemData(List<MegaTransfer> tL, int position) {
        this.tL = tL;
        notifyItemInserted(position);
    }

    /**
     * Removes a transfer from adapter.
     * Also checks if the transfer to remove is selected. If so, removes it from selected items list
     * and updates the list with the new positions that the rest of selected transfers occupies after
     * the removal.
     *
     * @param tL       Updated list of transfers.
     * @param position Item to remove.
     */
    public void removeItemData(List<MegaTransfer> tL, int position) {
        this.tL = tL;

        if (isItemChecked(position)) {
            int nextIndex = selectedItems.indexOfKey(position);
            selectedItems.delete(position);

            for (int i = nextIndex; i < selectedItems.size(); i++) {
                int pos = selectedItems.keyAt(i);
                selectedItems.delete(pos);
                selectedItems.append(pos - 1, true);
            }

            selectModeInterface.notifyItemChanged();
        }

        notifyItemRemoved(position);
    }

    /**
     * Moves a transfer.
     *
     * @param tL          Updated list of transfers.
     * @param oldPosition Old position of the transfer.
     * @param newPosition New position of the transfer.
     */
    public void moveItemData(List<MegaTransfer> tL, int oldPosition, int newPosition) {
        this.tL = tL;
        notifyItemMoved(oldPosition, newPosition);
    }

    /**
     * Updates the progress of a transfer.
     *
     * @param position Position of the transfer in the adapter.
     * @param transfer Transfer to which the progress has to be updated.
     */
    public void updateProgress(int position, MegaTransfer transfer) {
        ViewHolderTransfer holder = (ViewHolderTransfer) listFragment.findViewHolderForAdapterPosition(position);
        if (holder == null) {
            notifyItemChanged(position);
            return;
        }

        if (holder.progressText.getVisibility() == GONE) {
            holder.progressText.setVisibility(VISIBLE);
            holder.speedText.setVisibility(VISIBLE);
            holder.textViewCompleted.setVisibility(GONE);
            holder.imageViewCompleted.setVisibility(GONE);
        }

        holder.progressText.setText(getProgress(transfer));

        if (megaApi.areTransfersPaused(MegaTransfer.TYPE_DOWNLOAD)
                || megaApi.areTransfersPaused(MegaTransfer.TYPE_UPLOAD)) {
            holder.speedText.setText(context.getResources().getString(R.string.transfer_paused));
        } else {
            holder.speedText.setText(getSpeedString(isOnline(context) ?
                    transfer.getSpeed()
                    : 0));
        }

    }

    public boolean isMultipleSelect() {
        return multipleSelect;
    }

    public void setMultipleSelect(boolean multipleSelect) {
        if (this.multipleSelect != multipleSelect) {
            this.multipleSelect = multipleSelect;
            notifyDataSetChanged();
        }

        if (this.multipleSelect) {
            selectedItems = new SparseBooleanArray();
        } else if (selectedItems != null) {
            selectedItems.clear();
        }
    }

    public void hideMultipleSelect() {
        setMultipleSelect(false);
        selectModeInterface.destroyActionMode();
    }

    public void selectAll() {
        for (int i = 0; i < getItemCount(); i++) {
            if (!isItemChecked(i)) {
                toggleSelection(i);
            }
        }
    }

    public void clearSelections() {
        for (int i = 0; i < getItemCount(); i++) {
            if (isItemChecked(i)) {
                toggleSelection(i);
            }
        }
    }

    /**
     * Checks if the current position is selected.
     * If it is, deselects it. If not, selects it.
     *
     * @param pos Position to check.
     * @return True if the position is selected, false otherwise.
     */
    private boolean putOrDeletePosition(int pos) {
        if (selectedItems.get(pos, false)) {
            selectedItems.delete(pos);
            return true;
        } else {
            selectedItems.append(pos, true);
            return false;
        }
    }

    /**
     * Selects or deselects a transfer with an animation view.
     *
     * @param pos Position to select or deselect.
     */
    public void toggleSelection(int pos) {
        startAnimation(pos, putOrDeletePosition(pos));
        selectModeInterface.notifyItemChanged();
    }

    /**
     * Applies the animation to the select or deselect action.
     *
     * @param pos    Position of the view to animate.
     * @param delete True if the action is deselect, false if it is select.
     */
    private void startAnimation(final int pos, final boolean delete) {
        MegaTransfersAdapter.ViewHolderTransfer view = (MegaTransfersAdapter.ViewHolderTransfer) listFragment.findViewHolderForLayoutPosition(pos);
        if (view == null) {
            notifyItemChanged(pos);
            return;
        }

        Animation flipAnimation = AnimationUtils.loadAnimation(context, R.anim.multiselect_flip);
        flipAnimation.setAnimationListener(new Animation.AnimationListener() {
            @Override
            public void onAnimationStart(Animation animation) {
                if (!delete) {
                    notifyItemChanged(pos);
                }
            }

            @Override
            public void onAnimationEnd(Animation animation) {
                if (delete) {
                    notifyItemChanged(pos);
                }
            }

            @Override
            public void onAnimationRepeat(Animation animation) {

            }
        });

        view.imageView.startAnimation(flipAnimation);
    }

    /**
     * Checks if select mode is enabled. If so, checks if the position is selected.
     *
     * @param position Position to check.
     * @return True if the position is selected, false otherwise.
     */
    private boolean isItemChecked(int position) {
        return isMultipleSelect() && selectedItems != null
                && position >= 0
                && selectedItems.get(position);
    }

    public List<MegaTransfer> getSelectedTransfers() {
        if (selectedItems == null) {
            return null;
        }

        List<MegaTransfer> selectedTransfers = new ArrayList<>();

        for (int i = 0; i < selectedItems.size(); i++) {
            if (!selectedItems.valueAt(i)) {
                continue;
            }

            int selected = selectedItems.keyAt(i);
            if (selected < 0 || selected >= tL.size()) {
                continue;
            }

            MegaTransfer transfer = tL.get(selected);
            if (transfer != null) {
                selectedTransfers.add(transfer);
            }
        }

        return selectedTransfers;
    }

    public int getSelectedItemsCount() {
        if (selectedItems == null) {
            return 0;
        }

        return selectedItems.size();
    }

    @Override
    public List<Integer> getSelectedItems() {
        if (selectedItems == null) {
            return null;
        }

        List<Integer> items = new ArrayList<>(selectedItems.size());

        for (int i = 0; i < selectedItems.size(); i++) {
            items.add(selectedItems.keyAt(i));
        }

        return items;
    }

    @Override
    public int getFolderCount() {
        return 0;
    }

    @Override
    public int getPlaceholderCount() {
        return 0;
    }

    @Override
    public int getUnhandledItem() {
        return INVALID_POSITION;
    }

    public interface SelectModeInterface {
        void destroyActionMode();

        void notifyItemChanged();
    }

}
